%
%FMMA CRC Book/JRR/JS/ExampSolSW\Example 6.2Dryden

% load input data (de,xu,xw)

load inp.dat

% control input (doublet)

de = inp(:,1);

%----------------Constant parameters------------------%
Parameters

T  = 0.03;            % sampling time(sec)
N  = length(inp);     % number of data points

%-related to turbulence-%
Lu = 1750/3.29;       % meter
Lw = Lu;              % meter
%-----------------------------------------------------%

% Initial states of turbulenec parameters
yu   = 0;   
yq   = 0;   
yw1  = 0;
yw2  = 0;
xtrb = [yu yq yw1 yw2];

xptrb= [0 0 0 0];

% Random number generator to simulate random nature of turbulence
xu = inp(:,2);
xw = inp(:,3);


% Initial state of aircraft 

u  = 36.0;          % m/s
w  = 7.2;           % m/s
AOA= atan2(w,u);    % angle of attack in rad
Vt = sqrt(u^2+w^2); % m/s
q  = 0.0;           % rad/s
th = 0.15;          % rad
xta= [u w q th];

% Turbulence velocity in flight path & body axes

[ufturb,wfturb,qfturb,uturb,wturb,qturb]        = turbVel(xtrb,xptrb,Vt,Lu,Lw,sgt,b,AOA);

% Coeff Calc.
[Cx,Cz,Cm,qbar]                                 = coefficients(xta,[uturb wturb qturb],de(1));
k=1;

% Observations
[um(k),wm(k),qm(k),thm(k),axm(k),azm(k),qdm(k)] = observ(xta(k,:),[uturb(k) wturb(k) qturb(k)],qbar(k),Cx(k),Cz(k),Cm(k));


% Aircraft's Longitudinal Motion Simulation under Turbulence

for k=2:N
    
    time(k)                                                   = (k-1)*T;
    
    % State update of turbulence
    [xtrb(k,:),xptrb(k,:)]                                    = rk4int_turb(xtrb(k-1,:),xu(k-1),xw(k-1),Vt,Lu,Lw,sgt,b,T);
    
    % Turbulence velocity in flight path & body axes
    [ufturb(k),wfturb(k),qfturb(k),uturb(k),wturb(k),qturb(k)]= turbVel(xtrb(k,:),xptrb(k,:),Vt,Lu,Lw,sgt,b,AOA);
        
    % State update of aircraft
    [xta(k,:),xtap(k,:)]                                      = rk4int(xta(k-1,:),qbar(k-1),Cx(k-1),Cz(k-1),Cm(k-1),T);
    
    % Coeff Calc.
    [Cx(k),Cz(k),Cm(k),qbar(k)]                               = coefficients(xta(k,:),[uturb(k) wturb(k) qturb(k)],de(k));

    AOA                                                       = atan2(xta(k,2),xta(k,1));    
    Vt                                                        = sqrt(xta(k,1)^2+xta(k,2)^2); 
    
    % Observations
    [um(k),wm(k),qm(k),thm(k),axm(k),azm(k),qdm(k)]           = observ(xta(k,:),[uturb(k) wturb(k) qturb(k)],qbar(k),Cx(k),Cz(k),Cm(k));
end

