%
% FMMA/CRC Book/JRR/JS//ExampSolSW\Example 6.3Trim
% file name aircraftmm.m
% aircraft model for trim example of Chapter 6/Example 6.3
function [xd]=aircraftmm(time,x,u)

%-----------------  Constant  for C-880 ---------------%
s       =   2000.0;    % wing area   (ft^2)
cbar    =   18.94;     % mean chord  (ft)
mass    =   3.9167e3;  % mass        (lb)
iyy     =   2.45e6;    % intertia    (slug-ft^2)

tstat   =   6.0e4;   % static thrust
dtdv    =   -38.0;   % change in thrust w.r.t. change in velocity
ze      =   2.0;     % related to engine

cdcls   =   0.042;   % related to landing flaps, gears

% At 35000 ft and 0.8 Mach

cla     =   0.0838;  % per degree
cma     =  -0.0113;  % per degree
cmde    =  -0.0099;  % per degree
cmq     =  -0.0785;  % per rad
cmadot  =  -0.0785;  % per rad
cladot  =   0.0471;  % per rad
rtod    =   57.296;  % radian to degree
gd      =   32.17;   % gravitation constant ft/s^2

%--------------------------------------------------%

thtl    =   u(1);
elev    =   u(2); 
xcg     =   u(3);
land    =   u(4);
vt      =   x(1);
alpha   =   rtod*x(2);
theta   =   x(3);
q       =   x(4);
h       =   x(5);

% Atmospheric model

[mach,qbar]=Atm(h,vt);

qs=qbar*s;
salp=sin(x(2)); calp=cos(x(2));
gam=theta-x(2);sgam=sin(gam);cgam=cos(gam);

if land==0
    cl0=0.347;cd0=0.024;
    cm0=0.05;dcdg=0.0;dcmg=0.0;
elseif land==1
    cl0=1.0;cd0=0.08;
    cm0=-0.20;dcdg=0.02;dcmg=-0.05;
else
    disp('landing gear and flaps ? ')
end
thr=(tstat+vt*dtdv)*max(thtl,0);
cl=cl0+cla*alpha;
cm=dcmg+cm0+cma*alpha+cmde*elev+cl*(xcg-0.25);
cd=dcdg+cd0+cdcls*cl*cl;
xd(1)=(thr*calp-qs*cd)/mass-gd*sgam;
xd(2)=(-thr*salp-qs*cl+mass*(vt*q+gd*cgam))/(mass*vt+qs*cladot);
xd(3)=q;
d=0.5*cbar*(cmq*q+cmadot*xd(2))/vt;
xd(4)=(qs*cbar*(cm+d)+thr*ze)/iyy;
xd(5)=vt*sgam;
xd(6)=vt*cgam;
% END OF THE PROGRAM