%
% File name trim.m
% FMMA/CRC Book/JRR/JS/ExampSolSW\Example 6.3Trim
function UX = trim
global x u gamma

% ==============================================================
%  C-880 nonlinear model trimming routine                      
%  for longitudinal motion, steady level flight
% 
% Matlab command fminsearch is used to minimize the 
% cost function(cost = sum of weighted squared state derivatives)
% ===============================================================

% Initial Guess for free parameters
% ---------------------------------

UX0 = [u(1) u(2) x(2)];  % [throttle elevator alpha]

% Initializing optimization options and running optimization:
% -----------------------------------------------------------

OPTIONS = optimset('TolFun',1e-10,'TolX',1e-10,'MaxFunEvals',5e+04,'MaxIter',1e+04);

iter = 1;
while iter == 1
    
    [UX,FVAL] = fminsearch('cost',UX0,OPTIONS);
    cst       = cost(UX);
    
    x(2)=UX(3); x(3)=UX(3)+gamma;
    u(1)=UX(1); u(2)=UX(2);

    disp('Trim Values and Cost:');
    disp(['cost     = ' num2str(cst)])
    disp(['throttle = ' num2str(u(1))])
    disp(['elev     = ' num2str(u(2)) '(deg)'])
    disp(['alpha    = ' num2str(x(2)*180/pi) '(deg)'])

    flag = input('Continue trim routine iterations? (y/n):  ','s'); 
    if flag == 'n'
        iter = 0;
    end
    UX0 = UX;
end
    % END OF PROGRAM
