%   this file name is SFDIA.m* 
%  FMMA CRC BOOK/JRR/JS/ExampSolSW\Example 8.5KF
%-------------------------------------------------%
%       Parameters for Fault free simulation
%-------------------------------------------------%
N        = 100;
T        = 0.01;
A        =[-0.033  0.001   0.0   -9.81
            0.168  -0.387   260.0  0.00
            0.005  -0.0064 -0.55   0.00
            0.000   0.0     1.0    0.00]; 
B       = [ 0.45
            -5.18
            -0.91
             0.00];  
G       = eye(4,4);
H       = eye(4,4);
D       = zeros(4,1);

%-------------------------------------------------%
%       process noise simulation
%-------------------------------------------------%
Q   = 0.0001*eye(4,4);
for j=1:4
    randn('seed',j);
    w(:,j)  = randn(N,1);
    w(:,j)  = sqrt(Q(j,j))*(w(:,j)-mean(w(:,j)))/std(w(:,j));
end

%-------------------------------------------------%
%       Measurement noise simulation
%-------------------------------------------------%

randn('seed',5)

stdv = [0.06 0.01 0.10 0.05];

R    = diag(stdv.^2);
for j=1:4
    randn('seed',4+j);
    v(:,j)  = randn(N,1);
    v(:,j)  = stdv(j)*(v(:,j)-mean(v(:,j)))/std(v(:,j));
end

%-------------------------------------------------%
%       True inital state
%-------------------------------------------------%
xt(1,:)  = [0 0 0.05 0.06];;


%-------------------------------------------------%
%       Control input
%-------------------------------------------------%

u          = 0.5*ones(N,1);

%-------------------------------------------------%
%       Plant simulation
%-------------------------------------------------%
for k=2:N
    xt(k,:) = rk4int(xt(k-1,:),u(k),A,B,G,w(k,:),T);
end

%-------------------------------------------------%
%       Sensor simulation
%-------------------------------------------------%

for k=1:N
     time(k) = (k-1)*T;
        
    if k > 30 
        v(k,3) = v(k,3) + 0.4; % bias added to pitch rate measurement
    end 
    
    Zm(k,:) = (H*xt(k,:)'+ D*u(k) + v(k,:)')';
end

    %-------------------------------------------------%
    %       Parameters related to SFDIA
    %-------------------------------------------------%

    M        = 20;      % window length
    ChiTh    = 101.88;  % threshold for Ms = 80 (degree of freedom) and alpha = 0.95
    ChiThs   = 31.4;    % threshold for Ms = 19 (degree of freedom) and alpha = 0.95

    % degree of freedom (DOF) = number of measurement channels * M
    % in present case  DOF = 4*20 = 80
    % reference for ChiTh: Johnson R., Elementry Statistics, 6th edition, PWS, KENT
    % Publishing, Boston, 1992
    %-------------------------------------------------%
    %       Algorithm for SFDIA using RKF
    %-------------------------------------------------%

    xe        = 0.9*xt(1,:);
    pe        = diag((xe-xt(1,:)).^2);
    xest(1,:) = xe;
    yest(1,:) = (H*xe')';

    beta   = zeros(N,1);
    betams = zeros(N,4);
    betas  = zeros(N,4);
    mflag  = [0 0];
    for j=2:N
        
        [xe,pe,ye,S,Nu,dflag]= ekf(xe,pe,A,B,G,H,T,Zm(j,:),1.2*R,Q,u(j),mflag);
        xest(j,:)     = xe;
        yest(j,:)     = (H*xe')';
        Sb(j,:)       = sqrt(diag(S)');
        res(j,:)      = Nu;
        Nul(j,:)      = Nu;
        Sl(j,:,:)     = S;

        % Fault Detection algorithm 
        if j > M
            for k=j-M+1:j
                Sc      = reshape(Sl(j,:,:),4,4);
                beta(j) = beta(j) + Nul(k,:)*inv(Sc)*Nul(k,:)';
            end
            if beta(j) > ChiTh
                str1=num2str(j);
                str2='Fault detected at iteration number=';
                str = strcat(str2,str1);
               % disp(str);            
            else
                str1=num2str(j);
                str2='Fault free at iteration number=';
                str = strcat(str2,str1);
              %  disp(str);   
            end        
        else
            Sc      = reshape(Sl(j,:,:),4,4);
            beta(j) = Nul(j,:)*inv(Sc)*Nul(j,:)';
        end

        % Sensor Fault Isolation algorithm

        if j > M        
            for sch = 1:4
                for k=j-M+1:j
                    Sc      = Sl(j,sch,sch);
                    betams(j,sch) = betams(j,sch)+ Nul(k,sch)*inv(sqrt(Sc));
                end
                betams(j,sch)= (1/M)*betams(j,sch);
                for k=j-M+1:j
                    Sc      = Sl(j,sch,sch);
                    betas(j,sch) = betas(j,sch)+ (Nul(k,sch)*inv(sqrt(Sc))-betams(j,sch))^2;
                end
                betas(j,sch)= ((M-1)/(M-1))*betas(j,sch);

                if betas(j,sch) > ChiThs
                    str1=num2str(j);
                    str2=num2str(sch);
                    str3='Fault detected at iteration number and channel=';
                    str = strcat(str3,str1);
                    str = strcat(str,'|');
                    str = strcat(str,str2);
                  %  disp(str);       
                    
                    if rflag == 1
                    
                        % Removal of faulty channel done here

                        if sch==1
                            H     = [0 0 0 0
                                     0 1 0 0
                                     0 0 1 0
                                     0 0 0 1];                             
                        end
                        if sch==2
                            H     = [1 0 0 0
                                     0 0 0 0
                                     0 0 1 0
                                     0 0 0 1];                             
                        end
                        if sch==3
                            H     = [1 0 0 0
                                     0 1 0 0
                                     0 0 0 0
                                     0 0 0 1];                            
                        end
                        if sch==4
                            H     = [1 0 0 0
                                     0 1 0 0
                                     0 0 1 0
                                     0 0 0 0];                             
                        end                    
                        mflag = [1 sch];     
                    end
                    
                else
                    str1=num2str(j);
                    str2=num2str(sch);
                    str3='Fault free at iteration number and channel=';
                    str = strcat(str3,str1);
                    str = strcat(str,'|');
                    str = strcat(str,str2);
                   % disp(str);           
                end
               % pause(0.2);
            end
        end    
    end

