%   this file name is main.m* 
%  FMMA CRC BOOK/JRR/JS/ExampSolSW\Example 8.6AFDIA
%   MATLAB script to execute SFDIA
clc
disp('-----------------------------------------------------------')
disp('Actuator Fault Detection, Isolation and Reconfiguration')
disp('Using Model Based Approach - Augmented Kalman Filter')
disp('For Longitudinal Motion of an Aircraft')
disp('-----------------------------------------------------------')
clear all
close all
disp('Please Hit to Continue...')
pause

%-------------------------------------------------%
%      figure
%-------------------------------------------------%
fflag=1; % 1 for fig // 0 for no fig


%-------------------------------------------------%
%       Parameters for Fault free simulation
%-------------------------------------------------%

N        = 2000;

T        = 0.01;
A        =[-0.033  0.001   0.0   -9.81
            0.168  -0.387   260.0  0.00
            0.005  -0.0064 -0.55   0.00
            0.000   0.0     1.0    0.00]; 
B       = [ 0.45
            -5.18
            -0.91
             0.00];  
G       = eye(4,4);
H       = eye(4,4);
D       = zeros(4,1);

%-------------------------------------------------%
%       process noise simulation
%-------------------------------------------------%
Q   = 0.0001*eye(4,4);
for j=1:4
    randn('seed',j);
    w(:,j)  = randn(N,1);
    w(:,j)  = sqrt(Q(j,j))*(w(:,j)-mean(w(:,j)))/std(w(:,j));
end

%-------------------------------------------------%
%       Measurement noise simulation
%-------------------------------------------------%

randn('seed',5)
stdv = [0.06 0.01 0.10 0.05];

R    = diag(stdv.^2);
for j=1:4
    randn('seed',4+j);
    v(:,j)  = randn(N,1);
    v(:,j)  = stdv(j)*(v(:,j)-mean(v(:,j)))/std(v(:,j));
end

%-------------------------------------------------%
%       True inital state
%-------------------------------------------------%
xt(1,:)  = [0 0 0.05 0.06];;


%-------------------------------------------------%
%       Control input
%-------------------------------------------------%

u = 0.5*ones(N,1);

%-------------------------------------------------%
%       Plant simulation
%-------------------------------------------------%
for k=2:N
    xt(k,:) = rk4int(xt(k-1,:),u(k),A,B,G,w(k,:),T);
end


%-------------------------------------------------%
%       Sensor simulation
%-------------------------------------------------%

for k=1:N
    time(k) = (k-1)*T;
    Zm(k,:) = (H*xt(k,:)' + D*u(k) + v(k,:)')';
end

   
    %-------------------------------------------------%
    %       Parameters related to AFDIA
    %-------------------------------------------------%
    xti(1,:)=xt(1,:);
    
    %-------------------------------------------------%
    %       Impaired Plant simulation
    %-------------------------------------------------%
    
    Bis(1,:) = B';
    for k=2:N
        if k >= 100
            Bi = [B(1:2)' -0.45 B(4)]'; % Actuator fault added
        else
            Bi = B;
        end
        Bis(k,:) = Bi';
        xti(k,:) = rk4int(xti(k-1,:),u(k),A,Bi,G,w(k,:),T);
    end
    
    if fflag == 1
        str={'u(m/s)' 'w(m/s)' 'q(rad/s^2)' '\theta(rad)'};
        figure(1)
        for k=1:4
            subplot(2,2,k)
            plot(time,xt(:,k),'k')
            hold on;grid on
            plot(time,xti(:,k),'--k','Linewidth',2)
            if k==1 legend('true state-unimpaired','true state-impaired',0);end
            ylabel(str(k))
            if k==3 || k==4
                xlabel('Time [sec]')
            end
        end
    end
    
    %-------------------------------------------------%
    %       Sensor simulation
    %-------------------------------------------------%

    for k=1:N
        Zmi(k,:) = (H*xti(k,:)' + D*u(k) + v(k,:)')';
    end

    if fflag == 1
        figure(2)
        for k=1:4            
            subplot(2,2,k)
            plot(time,Zm(:,k),':k')
            hold on;grid on
            plot(time,Zmi(:,k),'.-k','MarkerSize',1)
            if k==1 legend('meas. state - unimpaired','meas. state - impaired',0);end
            ylabel(str(k))
            if k==3 || k==4
                xlabel('Time [sec]')
            end
        end
    end
    
    %-------------------------------------------------------------%
    %       Parameter estimation of vector B using augmented KF
    %-------------------------------------------------------------%

    Ha         = [H zeros(4,4)];
    xea        = 0*[xt(1,:) B']; xea(6)=B(2);
    pea        = diag((xea-[xt(1,:) B']).^2); 
    xaest(1,:) = xea;
    yaest(1,:) = (Ha*xea')';
    
    for j=2:N
        Aa = [A u(j)*eye(4,4);zeros(4,4) zeros(4,4)];      
        Ga = [G zeros(4,4);zeros(4,4) zeros(4,4)];
        Qa = [Q zeros(4,4);zeros(4,4) zeros(4,4)];
        [xea,pea,yea,S,Nu,dflag]= akf(xea,pea,Aa,Ga,Ha,T,Zmi(j,:),R,Qa); 
        xaest(j,:)     = xea;
    end

    if fflag == 1
        str={'B(1)' 'B(2)' 'B(3)' 'B(4)'};
        figure(3)
        for k=1:4
            subplot(2,2,k)
            plot(time,Bis(:,k),'--k','Linewidth',2)
            hold on;grid on
            plot(time,xaest(:,k+4),'k')
            if k==1 legend('true','est',0);end
            ylabel(str(k))
            if k==3 || k==4
                xlabel('Time [sec]')
            end
        end
    end


    %-------------------------------------------------%
    %       Plant simulation with state feedback
    %-------------------------------------------------%
    xts(1,:) = [0 0 0.05 0.06];

    QQ = zeros(4,4);
    QQ(1,1) = 0.01;
    QQ(2,2) = 0.00001;
    QQ(3,3) = 0.00001;
    QQ(4,4) = 0.00001;
    
    [K,S,E]=lqr(A,B,QQ,1);
    
    for k=2:N        
        us(k)    = -K*xts(k-1,:)';
        xts(k,:) = rk4int(xts(k-1,:),us(k),A,B,G,w(k,:),T);
    end
    
       
    %----------------------------------------------------%
    %      Impaired Plant simulation with state feedback
    %----------------------------------------------------%
    xtsi(1,:) = xts(1,:);
    
    for k=2:N   
        if k >= 100
            Bi = [B(1:2)' -0.45 B(4)]';
        else
            Bi = B;
        end
        usi(k)    = -K*xtsi(k-1,:)';
        xtsi(k,:) = rk4int(xtsi(k-1,:),usi(k),A,Bi,G,w(k,:),T);
    end
    
    
    %-------------------------------------------------%
    %      Reconfiguration of Impaired Plant 
    %-------------------------------------------------%
    xtsir(1,:) = xts(1,:);
    
    for k=2:N   
        Kr         = pinv(xaest(k,5:8)')*B*K;
        usir(k)    = -Kr*xtsir(k-1,:)';
        xtsir(k,:) = rk4int(xtsir(k-1,:),usir(k),A,xaest(k,5:8)',G,w(k,:),T);
    end
    
    str={'u(m/s)' 'w(m/s)' 'q(rad/s^2)' '\theta(rad)'};

    if fflag == 1
        figure(4)
        for k=1:4
            subplot(2,2,k)
            plot(time,xts(:,k),'k') 
            hold on;grid on
            plot(time,xtsi(:,k),'--k')   
            plot(time,xtsir(:,k),'dk','markerSize',2)   
            if k==1 legend('unimpaired','impaired','reconfigured',0);end
            ylabel(str(k))
            if k==3 || k==4
                xlabel('Time [sec]')
            end
        end
    end   
    
