%
% FMMA CRC BOOK/JRR/JS/ExampSolSW\Example 8.7ANFIS
clc
disp('-----------------------------------------------------------')
disp('Sensor Fault Detection, Isolation and Reconfiguration')
disp('Using Non Model Based Approach - ANFIS')
disp('For Longitudinal Motion of an Aircraft')
disp('-----------------------------------------------------------')
clear all
close all
disp('Please Hit to Continue...')
pause


%-------------------------------------------------%
%      figure
%-------------------------------------------------%
fflag=1; % 1 for fig // 0 for no fig


%-------------------------------------------------%
%       Parameters for Fault free simulation
%-------------------------------------------------%
N        = 100;

T        = 0.01;
A        =[-0.033  0.001   0.0   -9.81
            0.168  -0.387   260.0  0.00
            0.005  -0.0064 -0.55   0.00
            0.000   0.0     1.0    0.00]; 
B       = [ 0.45
            -5.18
            -0.91
             0.00];  
G       = eye(4,4);
H       = eye(4,4);
D       = zeros(4,1);

%-------------------------------------------------%
%       process noise simulation
%-------------------------------------------------%
Q   = 0.0001*eye(4,4);
for j=1:4
    randn('seed',j);
    w(:,j)  = randn(N,1);
    w(:,j)  = sqrt(Q(j,j))*(w(:,j)-mean(w(:,j)))/std(w(:,j));
end

%-------------------------------------------------%
%       Measurement noise simulation
%-------------------------------------------------%

randn('seed',5)

stdv = [0.06 0.01 0.10 0.05];

R    = diag(stdv.^2);
for j=1:4
    randn('seed',4+j);
    v(:,j)  = randn(N,1);
    v(:,j)  = stdv(j)*(v(:,j)-mean(v(:,j)))/std(v(:,j));
end

th = compTh(stdv,N,1000);

%-------------------------------------------------%
%       True inital state
%-------------------------------------------------%

xt(1,:)  = [0 0 0.05 0.06];;


%-------------------------------------------------%
%       Control input
%-------------------------------------------------%
randn('seed',1111);
u          = 0.5*ones(N,1);
u          = u + 0.0000000000001*randn(N,1);

%-------------------------------------------------%
%       Plant simulation
%-------------------------------------------------%
for k=2:N
    xt(k,:) = rk4int(xt(k-1,:),u(k),A,B,G,w(k,:),T);
end


%-------------------------------------------------%
%       Sensor simulation
%-------------------------------------------------%
ch = 1;

for k=1:N
    time(k) = (k-1)*T;

    Zm(k,:) = (H*xt(k,:)' + D*u(k) + v(k,:)')';
     
    if k > 30 
        v(k,1) = v(k,1) + 1.0; % bias added to 'u' measurement
    end 
    
    Zm1(k,:) = (H*xt(k,:)' + D*u(k) + v(k,:)')';
end


%-------------------------------------------------%
%       NN training
%-------------------------------------------------%

ind = [2 3 4
       1 3 4
       1 2 4
       1 2 3];

% ----  For sensor fault detection and isolation----%

%  for j=1:4
%      net(j)  = anfiss([u Zm(:,j)],xt(:,j));
%  end
load net

% ----  For sensor fault reconfigurtaion ----%
%  for j=1:4
%     netr(j)  = anfiss([u Zm(:,ind(j))],xt(:,j));
% end
load netr

str={'u(m/s)' 'w(m/s)' 'q(rad/s^2)' '\theta(rad)'};

for j=1:4
    
    for k = 1:N
        zme(k,j)= evalfis([u(k) Zm1(k,j)],net(j));
        % when true measurement (without fault known)
        %res = Zm(k,j)-zme(k,j);
        % when true measurement (without fault not known)
        res = Zm1(k,j)-zme(k,j);
       
        if res >= th(1,j) || res <= th(2,j)
             zmer(k,j)= evalfis([u(k) Zm1(k,ind(j))],netr(j));
             wlogic(k)=j;
        else
             zmer(k,j)=zme(k,j);
        end        
    end   
end

if fflag == 1
    
%     figure(1)
%     plot(time,ch*ones(N,1),'--k','MarkerSize',3)
%     hold on
%     plot(time,wlogic,'-dk','MarkerSize',3)
%     legend('Actual','Detected',0)
%     xlabel('Time [sec]')
%     ylabel('Detection Flag')
        
    figure(2)
    for j=1:4
        subplot(2,2,j)
        plot(time,xt(:,j),'k')
        hold on; grid on        
        plot(time,Zm1(:,j),':k')
        if j==ch
            plot(time,zmer(:,j),'-dk','MarkerSize',3) 
        else
            plot(time,zme(:,j),'-dk','MarkerSize',3)
        end
        if j==1 legend('true','meas.','est.-reconfigured',0);end
        ylabel(str(j))
        if j==3 || j==4
            xlabel('Time [sec]')
        end
    end

    for j=1:4
        figure(3)
        subplot(2,2,j)
        hold on;grid on
        if j==ch
            plot(time,Zm(:,j)-zmer(:,j),'dk','MarkerSize',3) 
            plot(time,Zm1(:,j)-zmer(:,j),'ok','MarkerSize',3) 
            legend('non faulty meas. minus reconfigured state','faulty meas. minus reconfigured state',0)
        else
            plot(time,Zm(:,j)-zme(:,j),'dk','MarkerSize',3) 
        end
        plot(time,th(1,j)*ones(N,1),'--k','MarkerSize',3)
        plot(time,th(2,j)*ones(N,1),'--k','MarkerSize',3)
        ylabel(str(j))
        if j==3 || j==4
            xlabel('Time [sec]')
        end
    end
end


