%
% FMMA CRC BOOK/JRR/JS/ExampSolSW\Example 9.3FKTT49rules
% Function for state and state error covarinace update %
% Inputs:-
% Xhat : States estimate (k-1/k-1) output of IMM/kf module ( dimension : 1x9 )
% F    : State transition matrix ( dimension : 9x9 )
% H    : Observation matrix ( dimension : 3x9 )
% Z    : Sensor data at kth point ( dimension : 3x1 )
% Phat : States error covariance estimate (k-1/k-1) output of IMM/kf module ( dimension : 9x9 )
% Q    : Process noise variance ( dimension : 3x3 )
% R    : Measurement noise variance ( dimension : 3x3 )
% G    : Noise driving matrix ( dimension : 3x9 )
% k    : Sampling point
%
% Ouputs:-
% Xhat : Updated states estimate (k/k) ( dimension : 1x9 )
% Phat : Updated state error covariance ( dimension : 9x9 )
% S    : Innovation covariance ( dimension : 3x3 )
% Nu   : Innovation sequence ( dimension : 3x1 )
%
% Intermediate variables:-
% Xtelda : Predicted states (k/k-1) ( dimension : 9x1 )
% Ptelda : Predicted states error covariance ( dimension : 9x9 )
% K      : Filter gain ( dimension : 9x3 )


function [Xhat,Phat,S,Nu,Ck] = fkf(Xhat,F,H,Z,Phat,Q,R,G,fis,past_Nu,T)

Xtelda = F*Xhat';
Ptelda = F*Phat*F'+G*Q*G';
S      = H*Ptelda*H'+R;	
K      = Ptelda * H'*inv(S);
Nu     = Z'-H*Xtelda;
Ck     = []; 
for j  = 1:length(Z)
    d_e  = (Nu(j)-past_Nu(j))/T;
    Cj   = evalfis([Nu(j) d_e],fis(j));
    Ck   = [Ck Cj];
end
Xhat   = (Xtelda + K*Ck')';
Phat   = (eye(length(H))-K*H)*Ptelda;

% End of function 