%
% FMMA CRC BOOK/JRR/JS/ExampSolSW\Example 9.3FKTT49rules
clc
disp('-----------------------------------------------------------')
disp('Fuzzy Kalman Filter for target tracking along x-y axes')
disp('Target model : Constant acceleration')
disp('Rules: (49) only')
disp('-----------------------------------------------------------')
clear all
close all
disp('Please Hit to Continue...')
pause

% Target & sensor data generation in X-Y axes direction

N  = 100; 

T  = 0.1;
Xt = [0 100 10 0 -100 -10];
F  = [1 T T^2/2; 0 1 T;0 0 1];
F  = [F zeros(3,3);zeros(3,3) F];
G  = [T^3/6 T^2/2 T]';
G  = [G' G']';
Q  = 0.0001;
R  = [100 0
      0    100];
H  = [1 0 0];
H  = [H zeros(1,3);zeros(1,3) H];

randn('seed',9876)
w  = randn(N,1);
w  = sqrt(Q)*(w-mean(w))/std(w);
randn('seed',6549)
v  = randn(N,2);
for j=1:2
    v(:,j)  = sqrt(R(j,j))*(v(:,j)-mean(v(:,j)))/std(v(:,j));
end

for i=1:N-1
    Xt(i+1,:) = (F*Xt(i,:)' + G*w(i))';
end
for i=1:N
    time(i) = (i-1)*T;
    Zm(i,:) = (H*Xt(i,:)' + v(i,:)')';
end

% Building Fuzzy Inference System (FIS) for fuzzy state correlator 
% used in Kalman filter 
% inputs : ex = innovation sequence, d_ex = (ex - past_ex)/timestep
% output : correction vector Ck 

fis_x = bldfiskf('fuzzy state correlator',R(1,1));
fis_y = bldfiskf('fuzzy state correlator',R(2,2));
fis   = [fis_x fis_y];
% Filter initialization

Xhat1    = Xt(1,:);
Phat1    = 10*eye(6);

Xe1(1,:) = Xhat1;
Pe1(1,:) = diag(Phat1)';

Nu1  = Zm(1,:)'-H*Xhat1'; 

Xhat2    = Xhat1;
Phat2    = Phat1;

Xe2(1,:) = Xhat2;
Pe2(1,:) = diag(Phat2)';

Nu2  = Zm(1,:)'-H*Xhat2'; 

for i=2:N
    [Xhat1,Phat1,S1,Nu1]    = kf(Xhat1,F,H,Zm(i,:),Phat1,Q,R,G);
    [Xhat2,Phat2,S2,Nu2,Ck] = fkf(Xhat2,F,H,Zm(i,:),Phat2,Q,R,G,fis,Nu2,T);
    Xe1(i,:) = Xhat1;
    Pe1(i,:) = diag(Phat1)';    
    Xe2(i,:) = Xhat2;
    Pe2(i,:) = diag(Phat2)';
    rsspe(i,1) = sqrt(sum((Xt(i,1)-Xe1(i,1)).^2 + (Xt(i,4)-Xe1(i,4)).^2)); 
    rsspe(i,2) = sqrt(sum((Xt(i,1)-Xe2(i,1)).^2 + (Xt(i,4)-Xe2(i,4)).^2));   
    rssve(i,1) = sqrt(sum((Xt(i,2)-Xe1(i,2)).^2 + (Xt(i,5)-Xe1(i,5)).^2)); 
    rssve(i,2) = sqrt(sum((Xt(i,2)-Xe2(i,2)).^2 + (Xt(i,5)-Xe2(i,5)).^2));     
    rssae(i,1) = sqrt(sum((Xt(i,3)-Xe1(i,3)).^2 + (Xt(i,6)-Xe1(i,6)).^2)); 
    rssae(i,2) = sqrt(sum((Xt(i,3)-Xe2(i,3)).^2 + (Xt(i,6)-Xe2(i,6)).^2));     
end


figure(1)
plot(time,rsspe(:,1),'-sk','MarkerSize',2)
hold on;grid on
plot(time,rsspe(:,2),'-dk','MarkerSize',3)
xlabel('Time [sec]')
ylabel('rsspe (m)')
legend('Kalman Filter','Fuzzy Kalman Filter',0)

figure(2)
plot(time,rssve(:,1),'-sk','MarkerSize',2)
hold on;grid on
plot(time,rssve(:,2),'-dk','MarkerSize',3)
xlabel('Time [sec]')
ylabel('rssve (m/s)')
legend('Kalman Filter','Fuzzy Kalman Filter',0)

figure(3)
plot(time,rssae(:,1),'-sk','MarkerSize',2)
hold on;grid on
plot(time,rssae(:,2),'-dk','MarkerSize',3)
xlabel('Time [sec]')
ylabel('rssae (m/s^2)')
legend('Kalman Filter','Fuzzy Kalman Filter',0)