%
%FMMA CRC BOOK/JRR/JS/ExampSolSW\Example 9.4DFKF
clc
disp('-----------------------------------------------------------')
disp('Kinematic consistency using UDEKF and DFKF')
disp('using Engineer in loop simulator data')
disp('-----------------------------------------------------------')
clear all
close all
disp('Please Hit to Continue...')
pause

% Initial state
Xt = [167 0.00101 17.305 0 0.10384 4000,0,0,0,0,0,1,1];
nx = 13;
nw = 13;
G  = eye(nw); 
q  = 1.0e-15;
Q  = q*eye(nw);
ny = 6;

% load flight data

load ggspdr.dat
[N,nc]=size(ggspdr);
w  = sqrt(q)*noise_gen(N,1111,nw);

% sampling time
T=0.025; % sec

v     = ggspdr(1,9);
alpha = ggspdr(1,2)/57.3;
beta  = ggspdr(1,3)/57.3;
phi   = ggspdr(1,7)/57.3;
theta = ggspdr(1,8)/57.3;
alt   = ggspdr(1,10);   
Zt(1,:)   = [v alpha beta phi theta alt]; % noise free measurements  

p  = ggspdr(1,4)/57.3;
q  = ggspdr(1,5)/57.3;
r  = ggspdr(1,6)/57.3;
ax = ggspdr(1,11)*9.78;
ay = ggspdr(1,12)*9.78;
az = -ggspdr(1,13)*9.78;
u(1,:)    = [p q r ax ay az];

for i=2:N
    p  = ggspdr(i,4)/57.3;
    q  = ggspdr(i,5)/57.3;
    r  = ggspdr(i,6)/57.3;
    ax = ggspdr(i,11)*9.78;
    ay = ggspdr(i,12)*9.78;
    az = -ggspdr(i,13)*9.78;
    u(i,:)    = [p q r ax ay az];
    % True states
    Xt(i,:) = rk4int(Xt(i-1,:),T,p,q,r,ax,ay,az,nx) + w(i,:);
    
    % Measurements
    v     = ggspdr(i,9);
    alpha = ggspdr(i,2)/57.3;
    beta  = ggspdr(i,3)/57.3;
    phi   = ggspdr(i,7)/57.3;
    theta = ggspdr(i,8)/57.3;
    alt   = ggspdr(i,10);   
    Zt(i,:)   = [v alpha beta phi theta alt]; % noise free measurements    
end
% MONTE CARLO simulation runs=2/ you can choose a higher number. 
for mc=1:2
    disp(strcat('Monte-Carlo Simulation No:',num2str(mc)))
    % Noisy Measurements
    randn('seed',1234 + mc)  
    v     = randn(N,ny);
    sig_s = std(Zt);
    snr   = 10;
    std_n = sig_s/sqrt(snr);

    for j = 1:ny
        v(:,j) = std_n(j)*(v(:,j)-mean(v(:,j)))/(std(v(:,j)));
    end

    for i=1:N
        time(i) = (i-1)*T;
        Zm(i,:) = Zt(i,:) + v(i,:);
    end
    R=diag(var(Zm-Zt));
    % State estimation using UDEKF and UKF 

    %READ THE OFFSET DISTANCES FROM NOSEBOOM TO CG
    
    xi=12.856;
    yi=0.011;
    zi=0.14;

    % Filters intialization

    % for UDEKF
    Xhat1    = Xt(1,:);
    % its covariance
    Phat1    = eye(nx);
    udf      = [Phat1 G];
    eps      = 1.0d-7;
    Xe1(mc,1,:) = Xhat1;
    ye1(mc,1,:) = meas(Xhat1,xi,yi,zi,u(1,:));
    Pe1(mc,1,:) = 2*diag(sqrt(Phat1))';
    err(mc,1,1) = sqrt(sum((Zt(1,:)-reshape(ye1(mc,1,:),1,ny)).^2));

    % % for DFKF

    Xhat2        = Xhat1;
    Phat2        = Phat1;
    % Needs to be tuned properly for optimal performance of UKF
    alpha        =  1;
    beta         =  0;
    kappa        =  0;

    Xe2(mc,1,:)     = Xhat2;
    ye2(mc,1,:)     = meas(Xhat2,xi,yi,zi,u(1,:));
    Pe2(mc,1,:)     = 2*diag(sqrt(Phat2))';
    err(mc,1,2)     = sqrt(sum((Zt(1,:)-reshape(ye2(mc,1,:),1,ny)).^2));
   
    for i=2:N

        [Xhat1,udf,S1,Nu1] = eudkf(Xhat1,udf,G,Q,R,T,Zm(i,:),u(i,:),xi,yi,zi,eps,nx,nw,ny);     Phat1       = ud2P(udf); 
        [Xhat2,Phat2,S2,Nu2] = ukf(Xhat2,Phat2,G,Q,R,T,Zm(i,:),u(i,:),xi,yi,zi,nx,nw,ny,alpha,beta,kappa);    

        Xe1(mc,i,:)   = Xhat1;
        ye1(mc,i,:)   = meas(Xhat1,xi,yi,zi,u(i,:));
        Pe1(mc,i,:)   = 2*diag(sqrt(Phat1))';  
        err(mc,i,1)   = sqrt(sum((Zt(i,:)-reshape(ye1(mc,i,:),1,ny)).^2));
        rbnd1(mc,i,:) = [Nu1 2*sqrt(diag(S1)')];
        Xe2(mc,i,:)   = Xhat2;
        ye2(mc,i,:)   = meas(Xhat2,xi,yi,zi,u(i,:));
        Pe2(mc,i,:)   = 2*diag(sqrt(Phat2))'; 
        err(mc,i,2)   = sqrt(sum((Zt(i,:)-reshape(ye2(mc,i,:),1,ny)).^2));
        rbnd2(mc,i,:) = [Nu2 2*sqrt(diag(S2)')];
    end
end

Xe1   = reshape(mean(Xe1),N,nx);
ye1   = reshape(mean(ye1),N,ny);
Pe1   = reshape(mean(Pe1),N,nx);
Xe2   = reshape(mean(Xe2),N,nx);
ye2   = reshape(mean(ye2),N,ny);
Pe2   = reshape(mean(Pe2),N,nx);
err   = reshape(mean(err),N,2);
rbnd1 = reshape(mean(rbnd1),N,2*ny);
rbnd2 = reshape(mean(rbnd2),N,2*ny);


% Plotting data
figure(1)
subplot(321)
plot(time,Zm(:,1),':k')
hold on;grid on
plot(time,ye1(:,1),'--sk','MarkerSize',1)
plot(time,ye2(:,1),'-ok','MarkerSize',2)
plot(time,Zt(:,1),'k')
ylabel('V, m/s')
legend('Measured','UDEKF','DFKF','True',0)
subplot(322)
plot(time,Zm(:,2),':k')
hold on;grid on
plot(time,ye1(:,2),'--sk','MarkerSize',1)
plot(time,ye2(:,2),'-ok','MarkerSize',2)
plot(time,Zt(:,2),'k')
ylabel('\alpha, rad')
subplot(323)
plot(time,Zm(:,3),':k')
hold on;grid on
plot(time,ye1(:,3),'--sk','MarkerSize',1)
plot(time,ye2(:,3),'-ok','MarkerSize',2)
plot(time,Zt(:,3),'k')
ylabel('\beta, rad')
subplot(324)
plot(time,Zm(:,4),':k')
hold on;grid on
plot(time,ye1(:,4),'--sk','MarkerSize',1)
plot(time,ye2(:,4),'-ok','MarkerSize',2)
plot(time,Zt(:,4),'k')
ylabel('\phi, rad')
subplot(325)
plot(time,Zm(:,5),':k')
hold on;grid on
plot(time,ye1(:,5),'--sk','MarkerSize',1)
plot(time,ye2(:,5),'-ok','MarkerSize',2)
plot(time,Zt(:,5),'k')
ylabel('\theta, rad')
xlabel('Time [sec]')
subplot(326)
plot(time,Zm(:,6),':k')
hold on;grid on
plot(time,ye1(:,6),'--sk','MarkerSize',1)
plot(time,ye2(:,6),'-ok','MarkerSize',2)
plot(time,Zt(:,6),'k')
ylabel('h, m')
xlabel('Time [sec]')

figure(2)
subplot(321)
plot(time,rbnd1(:,1),'sk','MarkerSize',2)
hold on;grid on;axis tight
plot(time,rbnd2(:,1),'ok','MarkerSize',2)
legend('UDEKF','DFKF',0)
plot(time,rbnd1(:,7),'--k')
plot(time,rbnd2(:,7),'--k')
plot(time,-rbnd1(:,7),'k')
plot(time,-rbnd2(:,7),'k')
ylabel('V, m/s')
subplot(322)
plot(time,rbnd1(:,2),'sk','MarkerSize',2)
hold on;grid on;axis tight
plot(time,rbnd2(:,2),'ok','MarkerSize',2)
plot(time,rbnd1(:,8),'--k')
plot(time,rbnd2(:,8),'--k')
plot(time,-rbnd1(:,8),'k')
plot(time,-rbnd2(:,8),'k')
ylabel('\alpha, rad')
subplot(323)
plot(time,rbnd1(:,3),'sk','MarkerSize',2)
hold on;grid on;axis tight
plot(time,rbnd2(:,3),'ok','MarkerSize',2)
plot(time,rbnd1(:,9),'--k')
plot(time,rbnd2(:,9),'--k')
plot(time,-rbnd1(:,9),'k')
plot(time,-rbnd2(:,9),'k')
ylabel('\beta, rad')
subplot(324)
plot(time,rbnd1(:,4),'sk','MarkerSize',2)
hold on;grid on;axis tight
plot(time,rbnd2(:,4),'ok','MarkerSize',2)
plot(time,rbnd1(:,10),'--k')
plot(time,rbnd2(:,10),'--k')
plot(time,-rbnd1(:,10),'k')
plot(time,-rbnd2(:,10),'k')
ylabel('\phi, rad')
subplot(325)
plot(time,rbnd1(:,5),'sk','MarkerSize',2)
hold on;grid on;axis tight
plot(time,rbnd2(:,5),'ok','MarkerSize',2)
plot(time,rbnd1(:,11),'--k')
plot(time,rbnd2(:,11),'--k')
plot(time,-rbnd1(:,11),'k')
plot(time,-rbnd2(:,11),'k')
ylabel('\theta, rad')
xlabel('Time [sec]')
subplot(326)
plot(time,rbnd1(:,6),'sk','MarkerSize',2)
hold on;grid on;axis tight
plot(time,rbnd2(:,6),'ok','MarkerSize',2)
plot(time,rbnd1(:,12),'--k')
plot(time,rbnd2(:,12),'--k')
plot(time,-rbnd1(:,12),'k')
plot(time,-rbnd2(:,12),'k')
ylabel('h, m')
xlabel('Time [sec]')

figure(3)
plot(time,err(:,1),'sk','MarkerSize',2)
hold on;grid on
plot(time,err(:,2),'ok','MarkerSize',2)
xlabel('Time [sec]')
ylabel('Root sum square observation error')
legend('UDEKF','DFKF',0)
pimv=100*norm(err(:,1)-err(:,2))/norm(err(:,1));
gtext(strcat('% improvement=',num2str(pimv)));

