%
% FMMA (CRC/JRR/JS)/ExampSolSW\Example 8.4FuzzyControl
lc
disp('-----------------------------------------------------------')
disp('Demonstration of Fuzzy Gain-scheduling')
disp('-----------------------------------------------------------')
clear all
close all
disp('Please Hit to Continue...')
pause

format long

global k1 k2

N  = 200;      % data points
T  = 0.1;      % sampling time interval
xd = [0 1];    % desired altitude
x  = [0.1 0.1];% initial state

xs = x;        % storage

% Control input
u = ones(N,1);

% Test Case 1
for k=2:N
    time(k) = (k-1)*T;
    for m = 1:2
        x(m) = rk4int(x(m),xd(m),u(k),T); 
    end
    xs(k,:)  = x;
end

% Test Case 2 (Fuzzy Logic)

k1 = 0.5;
k2 = 0.92;

FL_Gain_Sceduling = FLGS(0);

fxd = xd;        % desired state
fx  = [0.1 0.1]; % initial state
fxs = fx;        % storage

% for invariant system
ix  = [0.1 0.1];
ixs = ix;

k   = 1;
for m = 1:2
    w1(k,m)= gainComp(1,1,fx(m),FL_Gain_Sceduling);
    w2(k,m)= gainComp(2,1,fx(m),FL_Gain_Sceduling);
    u1(k,m)= fuzzy_control_law(fx(m),fxd(m),w1(k,m),w2(k,m));
end
        
for k=2:N
    for m = 1:2
        
        %---- for Fuzzy  system--------%
        fx(m)  = rk4int_fuzzy(fx(m),fxd(m),u1(k-1,m),w1(k-1,m),w2(k-1,m),T); 
        
        %---- for invariant system--------%
        ix(m)  = rk4int_ivs(ix(m),u(k-1),T); 
        %---------------------------------%
        
        w1(k,m)= gainComp(1,1,fx(m),FL_Gain_Sceduling);
        w2(k,m)= gainComp(2,1,fx(m),FL_Gain_Sceduling);
        u1(k,m)= fuzzy_control_law(fx(m),fxd(m),w1(k,m),w2(k,m));
    end
    fxs(k,:)  = fx;
    ixs(k,:)  = ix;
end

% Test case 3: bode diagram of open loop and close loop systems
sys1 = ss(-0.4,0.4,1,0); % at xd = 0
sys2 = ss(-2.5,2.5,1,0); % at xd = 1

%---close loop system ----%

% for xd=0
w1s=w1(N,1);w2s=w2(N,1); % at steady state

A    = -0.4*w1s-2.5*w2s;%0.4*k1*w1^2 + 2.5*k2*w2^2 + 2.5*k1*w1*w2 + 0.4*k2*w1*w2 -0.4*w1 - 2.5*w2;
B    = 0.4*w1s + 2.5*w2s;
C    = 1;
D    = 0;
sys3 = ss(A,B,C,D);

% for xd=1
w1s=w1(N,2);w2s=w2(N,2); % at steady state

A    = -0.4*w1s-2.5*w2s;%0.4*k1*w1^2 + 2.5*k2*w2^2 + 2.5*k1*w1*w2 + 0.4*k2*w1*w2 -0.4*w1 - 2.5*w2;
B    = 0.4*w1s + 2.5*w2s;
C    = 1;
D    = 0;
sys4 = ss(A,B,C,D);

% plots
figure(1)
plot(time,xs(:,1),'k','linewidth',2)
hold on
plot(time,xs(:,2),'--k','linewidth',2)
title('Response of system to unit step input (open-loop)')
legend('xd=0','xd=1',0)
axis([0 time(N) 0 1.2])
grid on
xlabel('Time [sec]')

figure(2)
plotmf(FL_Gain_Sceduling,'input',1)
hold
plotmf(FL_Gain_Sceduling,'input',2)
title('Gain-scheduling/fuzzy sets')
grid on
axis tight

figure(3)
plot(time,fxs(:,1),'k','linewidth',2)
hold on
plot(time,fxs(:,2),'--k','linewidth',2)
title('Response of system with Fuzzy feedback controller')
legend('xd=0','xd=1',0)
axis([0 time(N) 0 1.2])
grid on
xlabel('Time [sec]')
ylabel('Response')

figure(4)
plot(time,fxs(:,2),'--k','linewidth',2)
hold on
plot(time,ixs(:,2),'.-k','linewidth',2)
title('Comparison of IVS(Invariant system) and Fuzzy based system at xd=1')
legend('Fuzzy','IVS',0)
axis([0 time(N) 0 1.2])
grid on
xlabel('Time [sec]')
ylabel('Response')

figure(5)
bode(sys1)
hold
bode(sys2)
grid on
title('Bode digram of open loop system (-:xd=0, --:x=1)')

figure(6)
bode(sys3)
hold
bode(sys4)
grid on
title('Bode digram of close loop system at steady state(-:xd=0, --: xd=1)')
